#pragma once

#include<stdio.h>
#include<string.h>
#include<stdlib.h> // itoa
#include "Node.h"

class LongNumber
{
private:
	Node<int> *Lowest;
	unsigned int Count;
	unsigned int PowMod;

	Node<int> *GetReversedNodes()
	{
		Node<int> *highest = 0, *lowest = Lowest;
		while(lowest)
		{
			highest = new Node<int>(lowest->data, highest);
			lowest = lowest->next;
		}
		return highest;
	}

public:
	static const unsigned int Mod = 1000000000u, Pow = 9u;
	static const char *const Zeros;

	LongNumber() : Lowest(NULL), Count(0u), PowMod(0uL) {}
	LongNumber(const int value) : Lowest(new Node<int>(value)), Count(1u), PowMod(0u) { }
	LongNumber(const int *const array, const unsigned int count, unsigned int start = 0u) : Lowest(NULL), Count(0), PowMod(0uL)
	{
		unsigned long long r = 0;
		while(start < count)
		{
			r += array[start];
			Lowest = new Node<int>(r % Mod);
			r /= Mod;
			++Count;
			++start;
		}
	}
	~LongNumber()
	{
		Node<int> *temp = Lowest;
		while(Lowest)
		{
			Lowest = Lowest->next;
			temp->next = NULL;
			delete temp;
			temp = Lowest;
		}
		Count = 0u;
		PowMod = 0u;
	}

	char *ToDigitsCharArray()
	{
		if(!Lowest) return "0";
		Node<int> *highest = GetReversedNodes(), *temp = highest;
		char *array = new char[(Count + PowMod) * Pow + 1u],
			*num = new char[Pow + 1u];
		itoa(highest->data, num, 10);
		unsigned int index = strlen(num);
		memcpy(array, num, index);

		for(unsigned int i = 1u; i < Count; ++i)
		{
			highest = highest->next;
			delete temp;
			temp = highest;
			itoa(highest->data, num, 10);
			memcpy(array + index, Zeros, Pow);
			memcpy(array + (index += Pow) - strlen(num), num, Pow);
		}
		for(unsigned int i = 0u; i < PowMod; ++i)
		{
			memcpy(array + index, Zeros, Pow);
			index += Pow;
		}
		array[index] = '\0';
		delete[] num;
		return array;
	}

	LongNumber *operator *=(unsigned int n)
	{
		Node<int> *temp = Lowest;
		unsigned long long r = 0uL;
		if(temp)
		{
			while(temp->next)
			{
				r += (unsigned long long)temp->data * n;
				if(r % Mod)
				{
					temp->data = r % Mod;
					r /= Mod;
					temp = temp->next;
					break;
				}
				else
				{
					Lowest = Lowest->next;
					temp->next = NULL;
					delete temp;
					temp = Lowest;
					++PowMod;
					--Count;
				}
			}
			while(temp->next)
			{
				r += (unsigned long long)temp->data * n;
				temp->data = r % Mod;
				r /= Mod;
				temp = temp->next;
			}
			r += (unsigned long long)temp->data * n;
			temp->data = r % Mod;
			r /= Mod;
		}
		if(r > 0uL)
		{
			(Lowest ? temp->next : Lowest) = new Node<int>(r);
			++Count;
		}
		return this;
	}
};

const char *const LongNumber::Zeros = "000000000";
//*/
